#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Monitora se i computer dell'aula studenti sono accesi
# e restituisce delle statistiche a proposito.

import subprocess, time
from optparse import OptionsParser

# Qualche variabile globale
database_directory = '/home/robol/client_stats/' # Il traling slash ci serve
max_time = 86400 * 10 # 10 giorni

def LoadClients(group = 'all'):
    """
    Ritorna una lista con tutti i client all'interno del
    gruppo group di dsh. Il default è 'all'
    """
    clients = []
    for client in open('/etc/dsh/group/' + group, 'r'):
        client = client.strip()
        if client.strip != '' and client[0] != '#':
            clients.append (client)
    return clients

def IsAlive(client):
    """
    Ritorna True se il client è acceso e risponde ai ping
    """
    alive = subprocess.call(["fping", client])
    return (alive == 0)

def LoadClientData(client):
    """
    Carica i dati relativi a quel determinato client. Questi
    sono nella forma di un dizionario con gli oggetti time e
    Valori True o False
    """
    data = {}
    try:
        f = open(database_directory + client, 'r')
    except IOError:
        return data
    for line in f:
        (time, alive) = line.split(":")
        data[time] = alive
    f.close()
    return data

def DumpClientData(client, data):
    """
    Salva i dati relativi ad un client, eliminando quelli troppo
    vecchi
    """
    data = dict (filter(lambda i: (i[0] > time.time() - max_time),
                        data.items()))
    f = open(database_directory + client, 'w')
    for stat in data.items():
        f.write (":".join(map(str, stat)))
    f.close()

def UpdateClientsData(client, alive):
    """
    Aggiorna i dati relativi al client inserendo l'informazione
    che in questo momento è acceso.
    """
    data = LoadClientData(client)
    data[time.time()] = alive
    DumpClientData(client, data)

def PrintStats(client):
    """
    Stampa le statistiche sul client prescelto
    """
    data = LoadClientData(client)
    uptime = len(filter(lambda s: (s==True), data))/ float(len(filter))
    print "Uptime: %3.3f (%s)" % (100 * uptime, client)
    
    


if __name__ == "__main__":

    parser = OptionParser()
    parser.add_option("-c", "--check", dest="check", 
                      help = "Check if clients are up and update database",
                      action = "store_true", default=True)
    parser.add_option("-s", "--stats", dest="stats",
                      help = "Print stats about collected informations",
                      action = "store_true", default = False)
    
    (options, args) = parser.parse_args()

    if options.check:
        for client in LoadClients():
            UpdateClientsData(client, IsAlive(client))

    if options.stats:
        for client in LoadClients():
            PrintStats(client)
